package BarPlot;

# BoxPlot
#
# Original Author: Jonathan Fiscus 

#
# This software was developed at the National Institute of Standards and Technology by
# employees and/or contractors of the Federal Government in the course of their official duties.
# Pursuant to Title 17 Section 105 of the United States Code this software is not subject to 
# copyright protection within the United States and is in the public domain.
#
# "FactorAnalysis.pm" is an experimental system.
# NIST assumes no responsibility whatsoever for its use by any party.
#
# THIS SOFTWARE IS PROVIDED "AS IS."  With regard to this software, NIST MAKES NO EXPRESS
# OR IMPLIED WARRANTY AS TO ANY MATTER WHATSOEVER, INCLUDING MERCHANTABILITY,
# OR FITNESS FOR A PARTICULAR PURPOSE.

# $Id: BarPlot.pm,v 1.3 2010/02/22 13:48:19 jfiscus Exp $

use strict;

use MMisc;
use MErrorH;
use Data::Dumper;
use AutoTable;

sub new {
  my ($class) = @_;

  my $errormsg = new MErrorH("BoxPlot");

  my $self =  {
    atab => new AutoTable(),
    metricTitles => {},

    properties  => new PropList(),

    errormsg    => $errormsg,
  };

  ### Set the plot options
  $self->{properties}->addProp("horiz", "FALSE", ("FALSE", "TRUE"));
  $self->{properties}->addProp("col", "", ());


#  $self->{properties}->addProp("Box_ylim", "", ('\d+,\d+'));
#  $self->{properties}->addProp("Box_ylim", "", ('(-?\d*(\.\d*)?),(-?\d*(\.\d*)?)'));
#  $self->{properties}->addProp("lwd", "", ());
#  $self->{properties}->addProp("ylim", "", ());
#  $self->{properties}->addProp("cex", "", ());
#  $self->{properties}->addProp("cex.axis", "", ());
#  $self->{properties}->addProp("cex.lab", "", ());
#  $self->{properties}->addProp("mar", "", ());
#  $self->{properties}->addProp("las", "", ());
#print Dumper($self->{properties});
  bless $self;
#  print Dumper($self);
  return($self);
}

sub unitTest(){
  my $bp = new BarPlot();
  
  $bp->addData("Sys1", "value1", 10);
  $bp->addData("Sys1", "value2", 10);
  $bp->addData("Sys1", "value3", 10);

  $bp->addData("Sys2", "value1", 4);
  $bp->addData("Sys2", "value2", 10);
  $bp->addData("Sys2", "value3", 25);

  $bp->addData("Syssssss3", "value1", 4);
  $bp->addData("Syssssss3", "value2", 4);
  $bp->addData("Syssssss3", "value3", 2);

  print $bp->{atab}->renderTxtTable();
  $bp->setCol('c("red", "blue", "green")');
  $bp->setHorizontal("TRUE");
##  $bp->setLegendPosition(
  $bp->makePlot("foo");
  
}

sub addData(){
    my ($self, $barTitle, $metricTitle, $val) = @_;

#    print "$barTitle, $metricTitle, $val\n";

    my $index;
    if (exists($self->{metricTitles}->{$metricTitle})){
	$index = $self->{metricTitles}->{$metricTitle};
    } else {
	$self->{metricTitles}->{$metricTitle} = scalar(keys %{ $self->{metricTitles} });
	$index = $self->{metricTitles}->{$metricTitle};
    }

#    $self->{atab}->addData($barTitle, "title", "line".$tindex);
#    $self->{atab}->addData($val,      $metricTitle, "line".$tindex);

    $self->{atab}->addData($val, $barTitle, "line".$index);

#    print "addData($barTitle, title, line.$tindex);\n";
#    print "addData($val,      $metricTitle, line.$tindex)\n";
}


sub makePlot(){
    my ($self, $root) = @_;

    open (DFILE, "> $root.dat") || die "Error: Unable to open the data file\n";
    $self->{atab}->setProperties({ "KeyColumnCsv" => "Remove" });
    $self->{atab}->setProperties({ "SortRowKeyCsv" => "AsAdded" });
#    $self->{atab}->setProperties({ "SortColKeyCsv" => "asAdded" });
    print DFILE $self->{atab}->renderCSV(2);
    close(DFILE);

    my $titles = "c(\"".join("\",\"", sort {$self->{metricTitles}{$a} <=> $self->{metricTitles}{$b}} 
			   (keys %{ $self->{metricTitles} }))."\")";
    open (RFILE, "> $root.R") || die "Error: Unable to open R command file\n";
    print RFILE "data <- read.table(\"$root.dat\", sep=\",\", header=TRUE)\n";
    print RFILE "data\n";
    print RFILE "png(file='$root.png')\n";
    print RFILE "par(las=2)\n";
    print RFILE "par(mar=c(5,8,4,2))\n";
    print RFILE "barplot(as.matrix(data), legend = $titles";
    print RFILE ", horiz=".$self->{properties}->getValue("horiz");
    print RFILE ", col=".$self->{properties}->getValue("col") if ($self->{properties}->getValue("col") ne "");
    print RFILE ", axis.lty = 1)\n";  ### Adds the ???
    close RFILE;

    system "R --no-save < $root.R 2>&1 1> $root.R.log";

}

# $val must be either TRUE or FALSE
sub setHorizontal(){
    my ($self, $val) = @_;
    return $self->setProp("horiz", $val);
}

### colStr must be c("red", "blue", ...)
sub setCol(){
    my ($self, $colStr) = @_;
    return $self->setProp("col", $colStr);
}

##########

sub setProp(){
  my ($self, $prop, $val) = @_;
    
  if (! $self->{properties}->setValue($prop, $val)) {
    $self->_set_errormsg($self->{properties}->get_errormsg());
    return (0);
  }
  return (1);
}

sub getPropList(){
  my ($self) = @_;
  $self->{properties}->getPropList();
}

sub sanitize(){
    my($text) = @_;
    $text =~ s/[:_\-\.\/]//g;
    $text =~ s/#/N/;
    $text =~ s/%/Pct/;
    $text;
}
############################################################

sub _set_errormsg {
  my ($self, $txt) = @_;
  $self->{errormsg}->set_errormsg($txt);
}

#####

sub get_errormsg {
  my ($self) = @_;
  return($self->{errormsg}->errormsg());
}

#####

sub error {
  my ($self) = @_;
  return($self->{errormsg}->error());
}

#####

sub clear_error {
  my ($self) = @_;
  return($self->{errormsg}->clear());
}

#####

sub _set_error_and_return {
  my $self = shift @_;
  my $errormsg = shift @_;

  $self->_set_errormsg($errormsg);

  return(@_);
}

1;
